==> flink-1.14.0-src.tgz: flink-1.14.0/flink-table/flink-table-planner/src/main/scala/org/apache/flink/table/planner/plan/rules/logical/CalcSnapshotTransposeRule.scala <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.flink.table.planner.plan.rules.logical

import org.apache.flink.table.planner.plan.nodes.logical.{FlinkLogicalCalc, FlinkLogicalSnapshot}

import org.apache.calcite.plan.RelOptRule.{any, operand}
import org.apache.calcite.plan.{RelOptRule, RelOptRuleCall}
import org.apache.calcite.rex.RexOver

/**
  * Transpose [[FlinkLogicalCalc]] past into [[FlinkLogicalSnapshot]].
  */
class CalcSnapshotTransposeRule extends RelOptRule(
  operand(classOf[FlinkLogicalCalc],
    operand(classOf[FlinkLogicalSnapshot], any())),
  "CalcSnapshotTransposeRule") {

  override def matches(call: RelOptRuleCall): Boolean = {
    val calc = call.rel[FlinkLogicalCalc](0)
    // Don't push a calc which contains windowed aggregates into a snapshot for now.
    !RexOver.containsOver(calc.getProgram)
  }

  override def onMatch(call: RelOptRuleCall): Unit = {
    val calc = call.rel[FlinkLogicalCalc](0)
    val snapshot = call.rel[FlinkLogicalSnapshot](1)
    val newClac = calc.copy(calc.getTraitSet, snapshot.getInputs)
    val newSnapshot = snapshot.copy(snapshot.getTraitSet, newClac, snapshot.getPeriod)
    call.transformTo(newSnapshot)
  }
}

object CalcSnapshotTransposeRule {
  val INSTANCE = new CalcSnapshotTransposeRule
}

==> flink-1.17.2-src.tgz: flink-1.17.2/flink-table/flink-table-planner/src/main/scala/org/apache/flink/table/planner/plan/rules/logical/CalcSnapshotTransposeRule.scala <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.flink.table.planner.plan.rules.logical

import org.apache.flink.table.planner.plan.nodes.logical.{FlinkLogicalCalc, FlinkLogicalSnapshot}

import org.apache.calcite.plan.{RelOptRule, RelOptRuleCall}
import org.apache.calcite.plan.RelOptRule.{any, operand}
import org.apache.calcite.rex.RexOver

/** Transpose [[FlinkLogicalCalc]] past into [[FlinkLogicalSnapshot]]. */
class CalcSnapshotTransposeRule
  extends RelOptRule(
    operand(classOf[FlinkLogicalCalc], operand(classOf[FlinkLogicalSnapshot], any())),
    "CalcSnapshotTransposeRule") {

  override def matches(call: RelOptRuleCall): Boolean = {
    val calc = call.rel[FlinkLogicalCalc](0)
    // Don't push a calc which contains windowed aggregates into a snapshot for now.
    !RexOver.containsOver(calc.getProgram)
  }

  override def onMatch(call: RelOptRuleCall): Unit = {
    val calc = call.rel[FlinkLogicalCalc](0)
    val snapshot = call.rel[FlinkLogicalSnapshot](1)
    val newClac = calc.copy(calc.getTraitSet, snapshot.getInputs)
    val newSnapshot = snapshot.copy(snapshot.getTraitSet, newClac, snapshot.getPeriod)
    call.transformTo(newSnapshot)
  }
}

object CalcSnapshotTransposeRule {
  val INSTANCE = new CalcSnapshotTransposeRule
}
